//-----------------------------------------------------------------------------
/// \file	Main.cpp
/// \date	12-Jan-2005
/// \author	Rob Bateman
/// \brief	This time we will do something a little more interesting. We can
/// 		query the current viewport and then render an overlay into it.
/// 		using a global counter, we should be able to make simple openGL
/// 		animations....
//-----------------------------------------------------------------------------

// required to link the libraries
#ifdef WIN32
	#define NT_PLUGIN
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
#endif

// under WIN32 we have to 'export' these functions so that they are visible
// inside the dll. If maya can't see the functions, it can't load the plugin!
#ifdef WIN32
	#define EXPORT __declspec(dllexport)
#else
	#define EXPORT
#endif

#include <maya/MFnPlugin.h>
#include <maya/MEventMessage.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>
#include <maya/MPoint.h>
#include <maya/M3dView.h>
#include <time.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

/// Having created a callback function within maya, we need to store the generated
/// MCallbackId so that we can delete the callback.
///
MCallbackId g_Callback;

/// a global counter, used for y height
unsigned int count=0;

/// The function that performs our idle processing
///
void IdleFunc( void* /* ignore */ ) {

	// get the current viewport
	M3dView View = M3dView::active3dView();

	// Start drawing the overlay over the current view
	View.beginOverlayDrawing();

		// clear the overlay planes
		View.clearOverlayPlane();

		// Set up the orthographic projection matix
		glMatrixMode( GL_PROJECTION );
		glLoadIdentity();
		gluOrtho2D( 0.0f, View.portWidth(),
					0.0f, View.portHeight() );

		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();


		// translate our line accross the screen to the correct hieght
		glTranslatef(0,count++,0);
		glBegin(GL_LINES);
			glVertex2i(0,count);
			glVertex2i(View.portWidth(),count);
		glEnd();

		// make go back to the bottom
		if(count>View.portHeight())
			count = 0;

	// done drawing our overlay on the viewport
	View.endOverlayDrawing();
}




/// \brief	This function is called once when our plugin is loaded. We can use
///			it to register any custom nodes, mel functions etc.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus initializePlugin( MObject obj )
{
	// we need to use the plugin function set to register a new mel command with maya.
	// we can provide some info about the author of the plugin, the version number, and we
	// can also specify a required version of maya, though usually "Any" will do...
	//
	MFnPlugin fnPlugin( obj, "Rob Bateman", "1.0", "Any");

	/// When creating the callback for the idle function, we need to specify a callback
	/// type. In this case we are using "idle", however it can be anyone of the strings
	/// that `scriptJob -listEvents` returns;
	/// The second parameter is a pointer to the callback function. The third is a user
	/// data parameter (which is explained in MEventMessage2 example). In this case
	/// we can use NULL since we dont want any user data
	///
	g_Callback = MEventMessage::addEventCallback( "idle", IdleFunc, NULL );

	return MS::kSuccess;
}

/// \brief	This function is called once when our plugin is unloaded. We need
/// 		to tell maya which mel funcs, nodes etc we are removing.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus uninitializePlugin( MObject obj)
{
	// use the plugin function set to unload the plugin
	MFnPlugin fnPlugin( obj );

	// make sure we delete the callback before the plugin quits !!!
	// Maya is likely to crash otherwise!
	MEventMessage::removeCallback( g_Callback );

	// get the current viewport
	M3dView View = M3dView::active3dView();

	// clear the overlay before quitting the plugin
	View.beginOverlayDrawing();
		View.clearOverlayPlane();
	View.endOverlayDrawing();


	return MS::kSuccess;
}


